@echo off
REM ##########################################################################
REM # Install or Update Zabbix Agent for Windows - PLUS Version              #
REM # 2013 Bernhard Linz  /  Bernhard.Linz@datagroup.de                      #
REM #                        Bernhard@znil.de                                #
REM # http://znil.net                                                        #
REM #                                                                        #
REM # Download pre-compiled Agent from http://www.zabbix.com/download.php    #
REM # Unpack and place the 'bin' Folder in Directory of this Script          #
REM ##########################################################################
REM
REM Description: remove old Agent and install new one
REM              will use the %PROGRAMMFILES%\Zabbix Folder
REM              will use the right Version of Agent (32 or 64 Bit)
REM
REM History
REM V1.0    Simple Script
REM V1.1    with UserParameter servdisc.exe - Windows Service Autodiscover
REM V1.2    + UserParameter windowsupdates - Get the number of available Updates
REM V1.3    + Add Firewall-Rules for Zabbix Agent
REM V1.4    + Add UAC Permission Request if needed
REM
REM *** Configure the Agent **************************************************
REM 
REM DNS-Name or IP of the Zabbix-Server
Set ZABBIXSERVER=192.168.1.80
REM
REM
REM Port of Zabbix Server, Default = 10051
Set ZABBIXSERVERPORT=10051
REM
REM
REM Hostname to use
REM Computername ==> MYSERVER ************************************************
REM Set ZABBIXHOSTNAME=%COMPUTERNAME%
REM or Computername.domain.suffix ==> MYSERVER.testdomain.local **************
Set ZABBIXHOSTNAME=%COMPUTERNAME%
Set ZABBIXHOSTNAMESUFFIX=.%USERDNSDOMAIN%
REM
REM
REM Zabbix Host ListenPort, Default = 10050
Set ZABBIXHOSTLISTENPORT=10050
REM
REM
REM HostMetadata for Autodiscovery
REM Set a REM in front of the line if you did not use
REM Only use at Version 2.2 or higher, not for 2.0.x
REM Set ZABBIXHOSTMETADATA=Meisenweg
REM
REM Name of Config-File - Default zabbix_agentd.win.conf
Set ZABBIXHOSTCONFIGFILE=zabbix_agentd.win.conf
REM
REM ##########################################################################
REM # MAIN PROGRAM ###########################################################
REM ##########################################################################
REM Get the Path of this running Script
Set CURPATH=%~dp0
REM
REM Check for Administrator rights - needed for Install!!!
REM Solution found at: http://stackoverflow.com/questions/1894967/how-to-request-administrator-access-inside-a-batch-file
cls
Echo Administrative permissions required. Detecting permissions...
REM Next Command will fail if no admin-rights
net session >nul 2>&1
if %errorLevel% == 0 (
        REM we have a winner! ok, go on with work
        Echo Success: Administrative permissions confirmed.
        GoTo WITHADMINRIGHTS
    ) else (
        REM Nope, we need to request for the rights
        Echo Failure: Current permissions inadequate.
        Echo Please Re-Run this Script with Administrator Permissions!!!!
        Echo But wait a moment - i try to get the rights ...
        REM we create a .vbs at %TEMP% Folder - the vbs will restart this script with admin rights UAC request
        Echo Set UAC = CreateObject^("Shell.Application"^) > "%temp%\getadmin.vbs"
        Set params = %*:"=""
        Echo UAC.ShellExecute "%~s0", "%params%", "", "runas", 1 >> "%temp%\getadmin.vbs"
        "%temp%\getadmin.vbs"
        Exit /B
)
:WITHADMINRIGHTS
REM ok, we are running as admin, delete the UAC File if needed
if exist "%temp%\getadmin.vbs" ( del "%temp%\getadmin.vbs" )
REM Stop Agent if already Installed (=prepare for Update)
Echo Try to stop running Zabbix Agent - ignore Error-Messages
net stop "Zabbix Agent"
Echo Remove Zabbix old Zabbix Agent - ignore Error-Messages
"%PROGRAMFILES%\zabbix\zabbix_agentd.exe" -c "%PROGRAMFILES%\zabbix\zabbix_agentd.win.conf" --uninstall
Echo Delete old Files!
del "%PROGRAMFILES%\zabbix\zabbix_agentd.exe"
del "%PROGRAMFILES%\zabbix\zabbix_get.exe"
del "%PROGRAMFILES%\zabbix\zabbix_sender.exe"
Echo -------------------------------------------------------------------------
REM
REM Create Zabbix Folder if not exist ########################################
Echo.
Echo Create Folder '%PROGRAMFILES%\Zabbix'"
mkdir "%PROGRAMFILES%\Zabbix"
REM
REM
REM Mark the 32 and 64 Bit files with a File 32 or 64 Bit ####################
Echo 32Bit! > bin\win32\32Bit-Version!.txt
Echo 64Bit! > bin\win64\64Bit-Version!.txt
REM Testing for 32 or 64 Bit Version #########################################
Echo Check this Computer - %COMPUTERNAME% - for 32 or 64 Bit Architecture
If %PROCESSOR_ARCHITECTURE%==x86 (
  REM 32 Bit Windows!
  Echo This is a 32 Bit System, using bin\win32
  Echo Copy Files:
  Copy "%CURPATH%bin\win32\*" "%PROGRAMFILES%\Zabbix\" /Y

) else (
  REM 64 Bit Windows!
  Echo This is a 64 Bit System, using bin\win64
  Copy "%CURPATH%bin\win64\*" "%PROGRAMFILES%\Zabbix\" /Y
)
Echo -------------------------------------------------------------------------
REM Copy UserParameter Files
Echo Copy UserParameter Files ...
Copy "%CURPATH%UserParameter\*" "%PROGRAMFILES%\Zabbix\" /Y
Echo -------------------------------------------------------------------------
REM Copy Tool Files
Echo Copy Tools ...
Copy "%CURPATH%Tools\*" "%PROGRAMFILES%\Zabbix\" /Y
Echo -------------------------------------------------------------------------
REM
REM
REM Convert ZABBIXHOSTNAMESUFFIX to Lowercase:
setlocal enabledelayedexpansion
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:A=a!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:B=b!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:C=c!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:D=d!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:E=e!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:F=f!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:G=g!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:H=h!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:I=i!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:J=j!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:K=k!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:L=l!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:M=m!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:N=n!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:O=o!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:P=p!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:Q=q!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:R=r!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:S=s!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:T=t!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:U=u!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:V=v!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:W=w!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:X=x!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:Y=y!
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:Z=z!
REM and Spaces to Underline
set ZABBIXHOSTNAMESUFFIX=!ZABBIXHOSTNAMESUFFIX:^ =_!
setlocal disabledelayedexpansion
REM
REM Create Config-file #######################################################
Echo Creating the File: "%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo Existing File will be Override!!
REM The File!
REM ### Writing Header #######################################################
Echo # This is a config file for the Zabbix agent daemon (Windows)> "%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # To get more information about Zabbix, go to http://www.zabbix.com >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo. >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # Description of Parameters: >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # Version 2.0.x: https://www.zabbix.com/documentation/2.0/manual/appendix/install/windows_agent >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #                https://www.zabbix.com/documentation/2.0/manual/appendix/config/zabbix_agentd_win >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo. >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # Version 2.2.x: https://www.zabbix.com/documentation/2.2/manual/appendix/install/windows_agent >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #                https://www.zabbix.com/documentation/2.2/manual/appendix/config/zabbix_agentd_win >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo. >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###   ######   #######   ###  ###  ###  ###  ###   ########  >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###  ########  ########  #### ###  ###  #### ###  #########  >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###  ###  ###  ###  ###  ########  ###  ########  ###        >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###  ###  ###  ###  ###  ########  ###  ########  ###        >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###  ########  #######   ### ####  ###  ### ####  ### #####  >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###  ########  ######    ###  ###  ###  ###  ###  ### #####  >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###  ###  ###  ### ###   ###  ###  ###  ###  ###  ###   ###  >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ###  ###  ###  ###  ###  ###  ###  ###  ###  ###  ###  ###  ###   ###  >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo  #### ## ###   ##   ###  ##   ###   ##   ##   ##   ##   ##   ### ####  >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo   ## #  # #     #   # #   #   # #  ##    #   #    ##    #    ## ## #   >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo. >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # THIS FILE WAS CREATED BY AUTOMATED SETUP! SETTINGS MAYBE OVERWRITE AT >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # NEXT UPDATE! >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #######################################################################>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
REM ### Writing Parameter ####################################################
Echo ############ GENERAL PARAMETERS ##################>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo. >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo LogFile=%SYSTEMDRIVE%\zabbix_agentd.log >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo LogFileSize=5 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #	Specifies debug level >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #	0 - no debug >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #	1 - critical information >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #	2 - error information >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #	3 - warnings >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #	4 - for debugging (produces lots of information) >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo DebugLevel=3 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo EnableRemoteCommands=1 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo LogRemoteCommands=1 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo Server=%ZABBIXSERVER% >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ListenPort=%ZABBIXHOSTLISTENPORT% >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # ListenIP=0.0.0.0 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # StartAgents=3 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo ServerActive=%ZABBIXSERVER%:%ZABBIXSERVERPORT% >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo Hostname=%ZABBIXHOSTNAME%%ZABBIXHOSTNAMESUFFIX% >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo RefreshActiveChecks=120 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # BufferSend=5 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # BufferSize=100 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # MaxLinesPerSecond=100 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo Timeout=30 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # UnsafeUserParameters=0 >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
REM ### HostMetadata #########################################################
REM Write HostMetadata if there anyone ***************************************
If "%ZABBIXHOSTMETADATA%" == "" GoTo NOZABBIXMETADATA
Echo # For AutoRegistration >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo HostMetadata=%ZABBIXHOSTMETADATA% >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
:NOZABBIXMETADATA
REM
REM
REM ### Writing UserParameter ################################################
REM For Windows Service Discovery:
Echo UserParameter=service.discovery,"%PROGRAMFILES%\Zabbix\servdisc.exe" >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
REM For Check Windows Updates
Echo UserParameter=znil.windowsupdates,"%PROGRAMFILES%\Zabbix\windowsupdates.exe" >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
REM Just a Test
Echo UserParameter=znil.znilnettest,"%PROGRAMFILES%\Zabbix\znilnettest.exe" >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"

REM
REM
REM ### Writing Who did this #################################################
Echo. >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # This File was created with Zabbix-Agent-Installer Script>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # 2013 by Bernhard Linz -- Bernhard.Linz@datagroup.de>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #                          Bernhard@znil.net>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # Website:                 http://znil.net>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo # Latest Version at http://znil.net/index.php?title=Zabbix_Agent_f%C3%BCr_Windows_per_Skript/automatisch_installieren_-_Musterdatei>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #>>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #    ######## ##    ## #### ##           ##    ## ######## ######## >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #         ##  ###   ##  ##  ##           ###   ## ##          ##    >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #        ##   ####  ##  ##  ##           ####  ## ##          ##    >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #       ##    ## ## ##  ##  ##           ## ## ## ######      ##    >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #      ##     ##  ####  ##  ##           ##  #### ##          ##    >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #     ##      ##   ###  ##  ##       ### ##   ### ##          ##    >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo #    ######## ##    ## #### ######## ### ##    ## ########    ##    >>"%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%"
Echo Writing File Complete!
Echo -------------------------------------------------------------------------
REM
REM
REM Create Zabbix-Agent-Restart-Script
Echo Create Zabbix-Agent-Restart-Script ...
Echo @Echo off>"%PROGRAMFILES%\Zabbix\Restart-Zabbix-Agent.cmd"
Echo net stop "zabbix agent">>"%PROGRAMFILES%\Zabbix\Restart-Zabbix-Agent.cmd"
Echo net start "zabbix agent">>"%PROGRAMFILES%\Zabbix\Restart-Zabbix-Agent.cmd"
Echo Exit 0>>"%PROGRAMFILES%\Zabbix\Restart-Zabbix-Agent.cmd"
Echo -------------------------------------------------------------------------
REM Create znilTail.exe Script
Echo Create znilTail.exe call with the Zabbix Agent Logfile:
Echo @Echo off>"%PROGRAMFILES%\Zabbix\znilTail-with-zabbix_agentd.log.cmd"
Echo start "" /B "%PROGRAMFILES%\Zabbix\znilTail.exe" "%SYSTEMDRIVE%\zabbix_agentd.log">>"%PROGRAMFILES%\Zabbix\znilTail-with-zabbix_agentd.log.cmd"
Echo Exit 0>>"%PROGRAMFILES%\Zabbix\znilTail-with-zabbix_agentd.log.cmd"
Echo -------------------------------------------------------------------------
REM
REM Create Firewall Rules
Echo Create Firewall-Rules for Zabbix Agent
Echo Ignore Errors on Windows XP / 2003 or lower!
REM we would get Errors at Windows XP and maybe at Windows 2003 and lower (no advfirewall context) - but no matter
netsh advfirewall firewall add rule name="Zabbix agentd port %ZABBIXHOSTLISTENPORT% inbound" dir=in action=allow protocol=TCP localport=%ZABBIXHOSTLISTENPORT%
netsh advfirewall firewall add rule name="Zabbix agentd port %ZABBIXHOSTLISTENPORT% outbound" dir=out action=allow protocol=TCP localport=%ZABBIXHOSTLISTENPORT%
netsh advfirewall firewall add rule name="Zabbix to Server port %ZABBIXSERVERPORT% inbound" dir=in action=allow protocol=TCP localport=%ZABBIXSERVERPORT%
netsh advfirewall firewall add rule name="Zabbix to Server port %ZABBIXSERVERPORT% outbound" dir=out action=allow protocol=TCP localport=%ZABBIXSERVERPORT%
Echo -------------------------------------------------------------------------
REM Install Zabbix Service and Start Agent
Echo Install Zabbix-Service ...
"%PROGRAMFILES%\Zabbix\zabbix_agentd.exe" -c "%PROGRAMFILES%\Zabbix\%ZABBIXHOSTCONFIGFILE%" --install
Echo Starting the Zabbix Service ...
net start "zabbix agent"
Echo -------------------------------------------------------------------------
Echo ALL ACTION COMPLETED ! Bye Bye!
Echo -------------------------------------------------------------------------
REM remove next REM to read Messages if you run this Script with Doubleclick
REM pause